\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{ESGI}[2016/04/07 ESGI116 report class update]

%%
\usepackage{graphicx}
\usepackage{fancyhdr}% Required for titlepage construction in this file.
\usepackage{ifthen}% Required for logic in this file.
\usepackage[usenames]{color}% Presentation pictures do not work without this.
\usepackage{amssymb}% Extended maths symbols from the AMS.
\usepackage[english]{babel}

\newboolean{SGMode}
\setboolean{SGMode}{false} % Set to true by option SGreport

\DeclareRobustCommand{\numShort}{\paranoboxwidth=3em\renewcommand\theparano{(\arabic{section}.\arabic{parano})}}
\DeclareRobustCommand{\numLong}{\paranoboxwidth=4em\renewcommand\theparano{(\arabic{section}.\arabic{subsection}.\arabic{parano})}}
\DeclareRobustCommand{\alphaShort}{\paranoboxwidth=3em\renewcommand\theparano{(\Alph{section}.\arabic{parano})}}
\DeclareRobustCommand{\alphaLong}{\paranoboxwidth=4em\renewcommand\theparano{(\Alph{section}.\arabic{subsection}.\arabic{parano})}}
\DeclareRobustCommand{\resetParas}{\parano[1]}

%
%% Provision for error checks AtEndOfDocument
\newboolean{maketitleCalled}
\setboolean{maketitleCalled}{false}
%
\newcommand{\baseClass}{article}
\DeclareOption{SGreport}{%
  \setboolean{SGMode}{true}
}
\PassOptionsToClass{titlepage}{\baseClass} % Force a titlepage option
\DeclareOption*{\PassOptionsToClass{\CurrentOption}{\baseClass}}
\ProcessOptions\relax

\LoadClass{\baseClass}


  %%
  %% PDF Author and Title definitions
  %%
  \def\@pdftitle{}
  \newcommand{\pdftitle}[1]{\gdef\@pdftitle{#1}}
  \def\@pdfauthor{}
  \newcommand{\pdfauthor}[1]{\gdef\@pdfauthor{#1}}

  \usepackage{kvoptions-patch}% This patches premature expansion of parameterised options to hyperref, avoiding an infinite loop that prevents Latex compilation.
  \usepackage[pdftitle={\@pdfauthor},
    pdfauthor={\@pdftitle},
    bookmarksopen=true]{hyperref}
  
  % To remove the borders around hyperlinks and TOC entries with pdflatex:
  \hypersetup{% 
    pdfborder = {0 0 0}
  }

  \usepackage[nottoc]{tocbibind}  % Allows bibliography to appear in table of contents


% Required for \abstract command
\def\@author{}%{{\it Please use \textbackslash abstract command.}}
\renewcommand{\author}[1]{\gdef\@author{#1}}
\def\@abstract{}%{{\it Please use \textbackslash abstract command.}}
\renewcommand{\abstract}[1]{\gdef\@abstract{#1}}
% Required for \version command
\def\@version{}%{{\it Please use \textbackslash version command.}}
\newcommand{\version}[1]{\gdef\@version{#1}}
% Required for \date command
\def\@date{}%{{\it Please use \textbackslash date command.}}
\renewcommand{\date}[1]{\gdef\@date{#1}}
% Required for \hdrTitle command (Short title in header)
\def\@hdrTitle{}%{{\it Please use \textbackslash hdrTitle command for running header title.}}
\newcommand{\hdrTitle}[1]{\gdef\@hdrTitle{#1}}
\ifthenelse{\boolean{SGMode}}{% Used only in SG reports:
  % Required for \indSponsor command
  \def\@indSponsor{}%{{\it Please use \textbackslash indSponsor command.}}
  \newcommand{\indSponsor}[1]{\gdef\@indSponsor{#1}}
  % Required for \indAffiliation command
  \def\@indAffiliation{}%{{\it Please use \textbackslash indAffiliation command.}}
  \newcommand{\indAffiliation}[1]{\gdef\@indAffiliation{#1}}
  % Required for \uniSponsor command
  \def\@uniSponsor{}%{{\it Please use \textbackslash uniSponsor command.}}
  \newcommand{\uniSponsor}[1]{\gdef\@uniSponsor{#1}}
  % Required for \contributors command
  \def\@contributors{}%{{\it Please use \textbackslash contributors command.}}
  \newcommand{\contributors}[1]{\gdef\@contributors{#1}}
  % Required for \SGnum command (Study Group number)
  \def\@SGnum{}%{{\it Please use \textbackslash SGnum command.}}
  \newcommand{\SGnum}[1]{\gdef\@SGnum{#1}}
}{
  % Next, set unused fields to non-empty so they don't raise an error later:
  \def\@indSponsor{Non-empty field}
  \def\@indAffiliation{Non-empty field}
  \def\@uniSponsor{Non-empty field}
  \def\@contributors{Non-empty field}
  \def\@SGnum{Non-empty field}
}
%
% Next two definitions print the total number of pages on the front cover. They are based on the package lastpage.sty.
\def\lastpage@putlabel{\addtocounter{page}{-1}%
   \immediate\write\@auxout{\string
   \newlabel{LastPage}{{}{\thepage}{}{page.\thepage}{}}}%
   \addtocounter{page}{1}}
\def\lastSecPage@putlabel{
   \immediate\write\@auxout{\string
   \newlabel{LastSecPage}{{}{\thepage}{}{page.\thepage}{}}}%
   }
\AtEndDocument{%
   \message{AED: lastpage setting LastPage}%
   \clearpage\lastpage@putlabel%
   \ifthenelse{\boolean{maketitleCalled}}{     
   }{
     \ClassError{ESGI}{You seem to have forgetten to call \ maketitle }{\ maketitle should be called after all of the relevant title fields have been defined and before any body text. Please see a template for more information.}
   }
}%

  % Font definitions
  \newfont{\titlefont}{cmssbx10 at 20.736pt}
  \newfont{\authorfont}{cmssbx10 at 14.4pt}
  \newfont{\frontfont}{cmssbx10 at 12pt}
  \newfont{\tcfont}{cmssbx10 at 10pt}
  \newfont{\matlabfont}{cmtcsc10 at 12pt}
  %
  % Page layout parameters
  \setlength{\headheight}{14.5pt} \setlength{\voffset}{-1cm} \addtolength{\textheight}{2.3cm}
  \addtolength{\textwidth}{12mm} \addtolength{\oddsidemargin}{-3mm} \setlength{\footnotesep}{5mm}
  \setlength{\leftmarginiii}{1cm} \interfootnotelinepenalty=10000
  %\setlength\bibindent{-20em}

% Figures
\newlength{\parahang}
\newlength{\numberedParaLabelSep}
\newlength{\numberedParaLeftMargin}
\renewcommand{\topfraction}{1.0}
\renewcommand{\bottomfraction}{1.0}
\renewcommand{\textfraction}{0.0}

  \pagestyle{fancy}%
    \lhead{{\small \@hdrTitle}} \rhead {{\small ESGI\@SGnum}{\small ESGI\@SGnum}} \pagenumbering{roman}    

\pagenumbering{roman}
\setcounter{page}{1}
\renewcommand{\thepage}{\roman{page}}
\cfoot{\thepage} 
%
\if@titlepage%% BEGINNING OF TITLEPAGE
\renewcommand\maketitle{
  \setboolean{maketitleCalled}{true}% Check this AtEndDocument
  \begin{titlepage}%
    \thispagestyle{fancy} % Produces headers on the title page.
    \let\footnotesize\small
    \let\footnoterule\relax
    \let \footnote \thanks
    \null\vfil
    \ifx\@author \@empty
      \ClassError{ESGI}{Missing author specification - please use the \ author command before calling maketitle}{Please ensure all relevant titlepage fields are defined.}
    \fi
    \ifx\@abstract \@empty
      \ClassError{ESGI}{Missing abstract specification - please use the \ abstract command before calling maketitle}{Please ensure all relevant titlepage fields are defined.}
    \fi
    \ifx\@version \@empty
      \ClassError{ESGI}{Missing version specification - please use the \ version command before calling maketitle}{Please ensure all relevant titlepage fields are defined.}
    \fi
    \ifx\@date \@empty
      \ClassError{ESGI}{Missing date specification - please use the \ date command before calling maketitle}{Please ensure all relevant titlepage fields are defined.}
    \fi
    \ifx\@indSponsor \@empty
      \ClassError{ESGI}{Missing indSponsor specification - please use the \ indSponsor command before calling maketitle}{Please ensure all relevant titlepage fields are defined.}
    \fi
    \ifx\@indAffiliation \@empty
      \ClassError{ESGI}{Missing indAffiliation specification - please use the \ indAffiliation command before calling maketitle}{Please ensure all relevant titlepage fields are defined.}
    \fi
    \ifx\@uniSponsor \@empty
      \ClassError{ESGI}{Missing uniSponsor specification - please use the \ uniSponsor command before calling maketitle}{Please ensure all relevant titlepage fields are defined.}
    \fi
    \ifx\@contributors \@empty
      \ClassError{ESGI}{Missing contributors specification - please use the \ contributors command before calling maketitle}{Please ensure all relevant titlepage fields are defined.}
    \fi
    \ifx\@SGnum \@empty
      \ClassError{ESGI}{Missing SGnum specification - please use the \ SGnum command before calling maketitle}{Please ensure all relevant titlepage fields are defined.}
    \fi

  %% All necessary fields set if we get this far.
  %% FIRST PAGE OF TITLE PAGES
   \begin{center}%
    % Although the titlefont includes a size parameter, the \LARGE is necessary to ensure the correct \baselinestretch is applied at line breaks in the title. Note that \LARGE is the same size at the titlefont (20.74pt)
     {\LARGE \titlefont{\@title} \par}
       \vspace{1.5cm}
       {\frontfont Problem presented by}\\[4mm]
       {\large \@indSponsor} \\[4mm]
       {\it \@indAffiliation}
       \vfill
	\begin{figure}[h]
         \begin{center}
	        \includegraphics[width=0.3\linewidth,keepaspectratio=true]{esgi-durham}\\[10mm]
         \end{center}
       \end{figure}
 	\begin{center}
    {\bf ESGI\@SGnum~was jointly hosted by}\\
    \@uniSponsor\\
	Smith Institute for Industrial Mathematics and System Engineering\\[15mm]
	        \includegraphics[width=0.5\linewidth,keepaspectratio=true]{smith-durham}\\[10mm]
  \end{center}
	\lhead{{\small \@hdrTitle}} \rhead {{\small ESGI\@SGnum}} \pagenumbering{roman}    

\newpage

%% SECOND PAGE OF TITLE PAGES
~ \vspace{0.5cm}
  \begin{center}
    {\authorfont Report author}\\[2mm]
    \@author
  \end{center}
  \vspace{1cm}

       \begin{center}
         {\bf Executive Summary}
       \end{center}
       \begin{quote}
         \@abstract
       \end{quote}
	\vfill
     {\frontfont Version \@version \\ \@date} \\\pageref{LastSecPage}$+$\pageref{LastPage} pages\\      
	\vskip3mm     
   \end{center}\par

\newpage
  \begin{center}
    {\authorfont Contributors}\\[2mm]
    \@contributors
  \end{center}
  \vfill

\newpage
%% THIRD PAGE OF TITLE PAGES
\tableofcontents
\message{AED: lastpage setting LastSecPage}%
\lastSecPage@putlabel
\vfil\null
\end{titlepage}%% END OF TITLEPAGE

\pagenumbering{arabic} \setcounter{page}{1}

  \setcounter{footnote}{0}%
  \global\let\thanks\relax
  \global\let\maketitle\relax
  \global\let\@thanks\@empty
  \global\let\@author\@empty
  \global\let\@date\@empty
%  \global\let\@title\@empty
  \global\let\title\relax
  \global\let\author\relax
  \global\let\date\relax
  \global\let\and\relax

%  \global\let\@indSponsor\@empty
  \global\let\@uniSponsor\@empty
  \global\let\@indAffiliation\@empty
  \global\let\@contributors\@empty
%  \global\let\@SGnum\@empty
  \global\let\indSponsor\relax
  \global\let\uniSponsor\relax
  \global\let\indAffiliation\relax
  \global\let\contributors\relax
  \global\let\SGnum\relax

}

\fi

